package com.example.switchyard.switchyard_example;

import org.jboss.as.clustering.jgroups.ChannelFactory;
import org.jboss.as.clustering.jgroups.subsystem.ChannelFactoryService;
import org.jboss.as.clustering.jgroups.subsystem.ChannelService;
import org.jboss.as.naming.ManagedReferenceFactory;
import org.jboss.as.naming.ManagedReferenceInjector;
import org.jboss.as.naming.ServiceBasedNamingStore;
import org.jboss.as.naming.deployment.ContextNames;
import org.jboss.as.naming.service.BinderService;
import org.jboss.msc.service.ServiceActivator;
import org.jboss.msc.service.ServiceActivatorContext;
import org.jboss.msc.service.ServiceBuilder;
import org.jboss.msc.service.ServiceController;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.ServiceTarget;
import org.jboss.msc.value.InjectedValue;
import org.jgroups.Channel;
import org.jgroups.blocks.locking.LockService;

public class DistributedLockServiceActivator implements ServiceActivator {

    private static final String JNDI_NAME = "java:jboss/myapp/lock-service";
    private static final String LOCK_SERVICE_PREFIX = "lock-service";
    private static final String STACK_NAME = "udp";
    private static final String CHANNEL_NAME = "myapp-cluster";

    private ServiceName lockServiceName;

    @Override
    public void activate(ServiceActivatorContext context) {
        createChannel(context.getServiceTarget());
        bindChannelToJNDI(context.getServiceTarget());
    }

    void createChannel(ServiceTarget target) {
        InjectedValue<ChannelFactory> channelFactory = new InjectedValue<>();
        ChannelService channelService = new ChannelService(CHANNEL_NAME, channelFactory);
        ServiceName channelServiceName = ChannelService.getServiceName(CHANNEL_NAME);
        ServiceName stackServiceName = ChannelFactoryService.getServiceName(STACK_NAME);
        target.addService(channelServiceName, channelService)
                .addDependency(stackServiceName, ChannelFactory.class, channelFactory).install();

        lockServiceName = ServiceName.of(LOCK_SERVICE_PREFIX, CHANNEL_NAME);
        InjectedValue<Channel> channel = new InjectedValue<>();
        target.addService(lockServiceName, new DistributedLockService(channel, CHANNEL_NAME))
                .addDependency(ServiceBuilder.DependencyType.REQUIRED, channelServiceName, Channel.class, channel)
                .setInitialMode(ServiceController.Mode.ACTIVE).install();
    }

    void bindChannelToJNDI(ServiceTarget target) {
        ContextNames.BindInfo bindInfo = ContextNames.bindInfoFor(JNDI_NAME);
        BinderService binder = new BinderService(bindInfo.getBindName());
        ServiceBuilder<ManagedReferenceFactory> service =
                target.addService(bindInfo.getBinderServiceName(), binder);

        service.addAliases(ContextNames.JAVA_CONTEXT_SERVICE_NAME.append(JNDI_NAME));
        service.addDependency(lockServiceName, LockService.class, new ManagedReferenceInjector<LockService>(
                binder.getManagedObjectInjector()));
        service.addDependency(bindInfo.getParentContextServiceName(), ServiceBasedNamingStore.class,
                binder.getNamingStoreInjector());

        service.setInitialMode(ServiceController.Mode.PASSIVE).install();
    }
}
